Program Arvore_de_Huffman;

Uses Crt,Janela,Menu;

Const

 N = 36;                     { Numero de elementos Maximo }

Type

 APONTANODE = ^NODE;         { Apontador par anodo da arvore de Huffman }

 NODE       = Record         { Nodo da arvore de Huffman }
               FREQ: Word;   { Frequencia do nodo        }
               CDG : String; { Codigo correspondente     }
               LEFT,         { Apontador para filho a esquerda } 
               RIGHT,        { Apontador para filho a direita  }
               FATHER: APONTANODE{ apontador para nodo pai     }
              End;

 APONTAROOT = ^ROOTNODE;     { Apontador para arvore de prioridades }

 ROOTNODE   = Record            { Arvore de prioridades     }
               AP : APONTANODE; { Apontador para nodo folha da arvore de huffman }
               LEFT,            { Apontador para filho a esquerda }
               RIGHT: APONTAROOT{ Apontador para filho a direita  }
              End;

 STRN = STRING [N];
 FRQ  = Array [0..N-1] of Integer;   { Vetor que contera as frequencias }
 POS  = Array [0..N-1] of APONTANODE;{ Vetor de apontadores para os nodos folhas }
 COD  = Array [0..N-1] of STRN;      { Vetor que contera os codigos binarios }       
 ALP  = Array [0..N-1] of Char;      { Vetor que contera o alfabeto entrado  }

Var

 FREQUENCY: FRQ;
 POSITION: POS;
 CODE: COD;
 ALPH: ALP;
 ROOT: APONTANODE;            { Apontador para raiz da arvore }
 Nmax: Byte;                  { Numero de valores entrados }
 I: Word;                     { Contador Interno.    }

 Msge,
 Bitcod : String;

 Fundo         : Ponteiro_para_Janela; { Contera o fundo sobreposto }
 Menu_Principal: Menu_Texto_Vertical;  { Contera o menu principal   }
 Lista         : Listopcao;            { Contera uma lista de opcoes}
 Opcao         : Integer;              { Contera uma opcao do menu  }

{ Zera o coteudo dos vetores }
 Procedure Zere;

 Begin
  FillChar (FREQUENCY, Sizeof (Integer)*N, 0);
  FillChar (CODE, Sizeof (STRN) * N, 0);
  FillChar (ALPH, Sizeof (Char) * N, 0);
  FillChar (POSITION, Sizeof (APONTANODE) * N, 0);
 End;

{ Cria um novo nodo da arvore de Huffman, inicializando-o }
 Function MakeTree ( FREQ: Word; CODIGO: String): APONTANODE;

 Var

  P: APONTANODE;

 Begin
  New (P);
  P^.FREQ:=FREQ;
  P^.CDG:=CODIGO;
  P^.LEFT:=nil;
  P^.RIGHT:=nil;
  P^.FATHER:=nil;
  MakeTree:=P
 End;

{ Indica se nodo P e filho a esquerda(True) ou a direita(False)}
 Function IsLeft ( P: APONTANODE ): Boolean;

 Begin
  If P^.FATHER^.LEFT = P
   Then
    IsLeft:=True
   Else
    IsLeft:=False
 End;

{ Retorna o ponteiro para o nodo pai de P }
 Function Father ( P: APONTANODE ): APONTANODE;

 Begin
  Father:=P^.Father;
 End;

{ Seta P1 como filho a esquerda de P }
 Procedure SetLeft( P: APONTANODE; P1: APONTANODE);

 Begin
  P^.LEFT:=P1;
  P1^.FATHER:=P
 End;

{ Seta P2 como filho a direita de P }
 Procedure SetRight( P: APONTANODE; P2: APONTANODE);

 Begin
  P^.RIGHT:=P2;
  P2^.FATHER:=P
 End;

{ Acrescenta um bit a esquerda de um codigo binario }
 Procedure Acrescente (Var CODIGO: STRN; BIT: STRN);

 Var
  Aux: STRN;

 Begin
  Aux:=Concat (BIT, CODIGO);
  CODIGO:=Aux;
 End;

{ Insere um novo nodo na arvore de prioridades }
 Procedure Pqinsert ( Var TREE: APONTAROOT; NODE: APONTANODE );

 Var

  NOVO: APONTAROOT;

 Begin
  If TREE = nil      { Se arvore vazia, cria primeiro nodo }
   Then
    Begin
     New (NOVO);
     NOVO^.AP:=NODE;
     NOVO^.LEFT:=nil;
     NOVO^.RIGHT:=nil;
     TREE:=NOVO
    End
   Else
    Begin
     If NODE^.FREQ >= TREE^.AP^.FREQ
      Then
       If TREE^.RIGHT = nil { Se a frequencia a ser inserida for menor que a do}
        Then                { nodo Tree corrente, e o nodo a direita for nil,  }
         Begin              { entao insere o nodo a direita, senao chama o pro-}
          New (NOVO);       { cedimento recursivamente com um novo nodo Tree   } 
          NOVO^.AP:=NODE;   { Se a frequencia for identica a outra ja na arvore}
          NOVO^.LEFT:=nil;  { esta nova sera inserida a direita tambem         }
          NOVO^.RIGHT:=nil;
          TREE^.RIGHT:=NOVO
         End
        Else
         Pqinsert (TREE^.RIGHT, NODE)
      Else
       If TREE^.LEFT = nil  { Se a freq. a ser inserida for menor, deve ser in-} 
        Then                { serida a esquerda. Tambem chamamos pqinsert      }
         Begin              { recursivamente, ate achar o local ideal para a   }
          New (NOVO);       { insercao do novo elemento }
          NOVO^.AP:=NODE;
          NOVO^.LEFT:=nil;
          NOVO^.RIGHT:=nil;
          TREE^.LEFT:=NOVO
         End
        Else
          Pqinsert ( TREE^.LEFT, NODE )
    End
 End;

{ Procura e deleta o nodo de menor valor da arvore de Prioridades }
 Function PqMinDelete ( Var TREE: APONTAROOT ): APONTANODE;

 Var

  APONTAUX: APONTAROOT;

Begin
 If TREE = nil
  Then
    Writeln('  ERRO : Fila de prioridades vazia !!  ')
  Else
   If (TREE^.LEFT = nil) and (TREE^.RIGHT = nil)
    Then   { Deleta o nodo raiz se os nodos filhos forem nil }
     Begin
      PqMinDelete:=TREE^.AP;
      Dispose(TREE);
      TREE:=nil
     End
    Else
     Begin
      If TREE^.LEFT = nil { Caso Especial quando o nodo raiz e o menor }
       Then               { ele e deletado e o filho a direita passa a }
        Begin             { ser o nodo raiz da arvore }
         APONTAUX:=TREE;
         TREE:=TREE^.RIGHT;
         PqMinDelete:=APONTAUX^.AP;
         Dispose(APONTAUX)
        End
       Else               { Neste caso, o menor se encontra a esquerda }
        Begin             { Se o proximo for nil, entao este e o menor,}
         If TREE^.LEFT^.LEFT = nil { e deve ser deletado }
          Then
           Begin
            APONTAUX:=TREE^.LEFT;
            TREE^.LEFT:=TREE^.LEFT^.RIGHT;
            PqMinDelete:=APONTAUX^.AP;
            Dispose(APONTAUX)
           End
          Else            { A funcao e chamada recursivamente ate achar }
            PqMinDelete:=PqMinDelete(TREE^.LEFT);{ o nodo de menor freq.}
     End;
  End;
End;

{ codifica um codigo recebido em Msge num string binario Bitcod }
Procedure Encode(Alph:ALP; Code:COD; Msge:String;Var Bitcod:String);

Var S    :String;
   cont  :Integer;
   Codigo:String;
   Erro  :Boolean;

Begin
 Erro:=False;
 Bitcod:='';
 S:=Copy(Msge,1,1);
 While (S<>'') do
  Begin
   Delete(Msge,1,1);
   Codigo:='';             { Busca a letra, e coloca o codigo correspondente }
   For Cont:=0 to Nmax-1 do{ em Bitcod, caso ache.Senao,seta mensagem de erro}
       If S=Alph[Cont] Then Codigo:=Code[Cont];
   If Codigo = '' Then Erro:=True
                  Else Bitcod:=Concat(Bitcod,Codigo);
   S:=Copy(Msge,1,1);
  End;
  If Erro Then Write(' ERRO: Mensagem Invalida! ')
End;

{ Decodifica um string binario Bitcod em uma mensagem Msge }
Procedure Decode(Alph:ALP; Code:COD;Var Msge:String; Bitcod:String);

Var S,g,
    Codigo:String;
    Cont,
    Conta :Integer;
    Erro  :Boolean;

Begin
 Erro:=False;
 Msge:='';
 Cont:=1;
 Codigo:='';
 While Bitcod<>'' do
  Begin
   Repeat
    g:=Copy(Bitcod,1,1);{ Pega um bit, procura na tabela e coloca em Msge  }
    S:=Concat(S,g);     { o caractere correspondente em Msge. Caso nao ache}
    Cont:=Cont+1;       { seta mensagem de erro }
    Delete(Bitcod,1,1);
    For Conta:=0 to Nmax-1 do
        If S=Code[Conta] Then Codigo:=Alph[Conta];
    If Codigo <> '' Then Msge:=Concat(Msge,Codigo);
    If (Cont>=Nmax) and (Codigo='') Then Erro:=True
   Until (Codigo<>'') or (Bitcod='');
  Codigo:='';
  S:='';
  g:='';
  Cont:=1;
  End;
 If Erro Then Write(' ERRO: Mensagem Invalida! ')
End;

{ Cria uma arvore de Huffman para o alfabeto corrente em Alph }
Procedure Huffman(Var ROOT : APONTANODE);
Var
 ROOTNODES : APONTAROOT;
 P,
 P1,
 P2        : APONTANODE;

Begin
 ROOTNODES:=nil;
 For I:=0 to (Nmax-1) do
  Begin
   P:=MakeTree (Frequency [I],Alph[I]);
   Position [I]:=P;
   Pqinsert (ROOTNODES, P)
  End;
 While ( ROOTNODES^.LEFT <> nil ) or ( ROOTNODES^.RIGHT <> nil ) do
  Begin
   P1:=Pqmindelete (ROOTNODES);
   P2:=Pqmindelete (ROOTNODES);
   P:=MakeTree ((P1^.FREQ) + (P2^.FREQ),Concat(P1^.CDG,P2^.CDG));
   SetLeft (P, P1);
   SetRight (P, P2);
   Pqinsert (ROOTNODES,P);
  End;
  ROOT:=pqmindelete (ROOTNODES);
  For I:=0 to (Nmax-1) do
   Begin
    P:=Position [I];
    code [I]:='';
    While (P <> ROOT) do
     Begin
      If ( isleft (P) )
       Then ACRESCENTE (CODE[I], '0')
       Else ACRESCENTE (CODE[I], '1');
      P:=Father (P)
     End;
   End;
End;

Procedure Pause;
Var Ch:Char;
Begin
 Moldura(21,20,23,60,1);
 Clrscr;
 Write('  Digite Algo Para Continuar : ');
 Repeat Until Keypressed;
 Ch:=ReadKey;
End;


Procedure Codifica(Msge:String;Var Bitcod:String);
Begin
 GotoXY(10,7);
 Writeln(' Digite uma mensagem a ser codificada : ');
 Moldura(9,3,12,77,1);
 Write(' Mensagem : ');
 Readln(Msge);
 Encode(Alph, Code, Msge, Bitcod);
 Moldura(14,3,17,77,1);
 Write(' Codigo : ');
 Write(Bitcod);
 Pause;
End;

Procedure Decodifica(Var Msge:String; Bitcod:String);
Begin
 GotoXY(10,7);
 Writeln(' Digite um codigo binario a ser decodificado : ');
 Moldura(9,3,12,77,1);
 Write(' Codigo : ');
 Readln(Bitcod);
 Decode(Alph, Code, Msge, Bitcod);
 Moldura(14,3,17,77,1);
 Write(' Mensagem : ');
 Write(Msge);
 Pause;
End;

Procedure Entra_Dados(Var Frequency:FRQ; Var Alph:ALP; Var Nmax:Byte);
Var Cont: Integer;
    N   : String;
    Naux: Integer;
Begin
{$I-}
 Moldura(5,1,24,80,1);
 Clrscr;
 GotoXY(7,17);
 Write(' Digite um numero de 2 a 36 ou <Enter> para retornar. ');
 Repeat
  GotoXY(49,3);
  Write('                 ');
  GotoXY(13,3);
  Write(' Quantos Dados Deseja Entrar ? (max 36) ');
  Readln(N);
  If N='' Then Exit;
  Val(N,Naux,Cont);
 Until (Naux>1) and (Naux<=36) and (IOResult=0);
 Nmax:=Naux;
 Moldura(5,1,24,80,1);
 Clrscr;
 For Cont:=0 to Nmax-1 do
  Begin
   Writeln;
   Repeat
     GotoXY(2,WhereY);
     Write(Cont+1:5);
     GotoXY(10,WhereY);
     Write('Caractere : ');
     Readln(Alph[Cont]);
   Until IoResult=0;
   Repeat
     Writeln;
     GotoXY(35,WhereY-2);
     Write('Frequencia :           ');
     GotoXY(48,WhereY);
     Readln(Frequency[Cont]);
   Until IOResult=0;
  End;
{$I+}
 Huffman(ROOT);
End;

Procedure Liste(Frequency:FRQ; Alph:ALP; Nmax:Byte);
Var Contlin,
    Conta : Integer;

Begin
Contlin:=0;
Repeat
 Conta:=0;
 Moldura(5,1,24,80,1);
 Clrscr;
 Writeln;
 While(Conta<12) do
  Begin
   If (Contlin<=Nmax-1) Then
    Writeln(Contlin+1:10,'        Caractere  :  ',Alph[Contlin],
            '       Frequencia  :  ',Frequency[Contlin]);
   Inc(Conta);
   Inc(Contlin)
  End;
 Pause;
Until Contlin>=Nmax;
End;

{ Procedimento recursivo que imprime a arvore formada }
Procedure PrintTree(R:APONTANODE;X,Y,Prof:Byte);
Begin
 GotoXY(X,Y);
 Write(R^.Freq);
 if R^.Right<>Nil then PrintTree(R^.Right,X+Prof,Y+2,Prof-3);
 if R^.Left<>Nil then PrintTree(R^.Left,X-Prof,Y+2,Prof-3);
 GotoXY(X,Y+1);
 Write(R^.CDG);
End;


Procedure Arvore(ROOT : APONTANODE);
Var X,Y :Byte;
Begin
 Moldura(5,1,24,80,1);
 Clrscr;
 X:=37;
 Y:=1;
 PrintTree(ROOT,X,Y,14);
 GotoXY(1,16);
 Pause;
End;

Procedure Saida;
Begin
 Menu_Principal.Destroi;
 GotoXY(1,25);
 Feche_Janela(Fundo);
 Halt;
End;

Begin
 ZERE;
 Frequency[0]:=15;
 Frequency[1]:=6;
 Frequency[2]:=7;
 Frequency[3]:=12;
 Frequency[4]:=25;
 Frequency[5]:=4;     { Tabela de frequencias e alfabeto Default }
 Frequency[6]:=6;
 Frequency[7]:=1;
 Frequency[8]:=15;
 Alph[0]:='A';
 Alph[1]:='B';
 Alph[2]:='C';
 Alph[3]:='D';
 Alph[4]:='E';
 Alph[5]:='F';
 Alph[6]:='G';
 Alph[7]:='H';
 Alph[8]:='I';
 Nmax:=9;            { Numero de entradas na tabela }
 Huffman(ROOT);
 Salve_Janela(1,1,24,80,Fundo);
 ClrScr;
 Moldura(1,1,4,80,1);
 TextColor(10);
 Writeln('                      >>>       HUFFMAN      <<<');
 TextColor(15);
 Write('              Programa para Codificar/Decodificar Mensagens ');
 Lista:=Vazio;
 Lista[1]:=' Entrar Tabela ';
 Lista[2]:=' Codificar     ';
 Lista[3]:=' Decodificar   ';
 Lista[4]:=' Listar Tabela ';
 Lista[5]:=' Listar Arvore ';
 Lista[6]:=' Sair p/ DOS   ';
 Menu_Principal.Init(30,9,Lista);
Repeat
 Moldura(5,1,24,80,1);
 Clrscr;
 GotoXY(21,17);
 Writeln('  Faca sua Escolha e Digite <Enter>.');
 Menu_Principal.Mostra;
 Opcao:=Menu_Principal.Opcao;
 Menu_Principal.Apaga;
 Case Opcao of
   1: Entra_Dados(Frequency,Alph,Nmax);
   2: Codifica(Msge,Bitcod);
   3: Decodifica(Msge,Bitcod);
   4: Liste(Frequency,Alph,Nmax);
   5: Arvore(ROOT);
 End;
Until Opcao = 6;
Saida;
End.